/*
 * Copyright (c) 2018 Institute for Integrated Circuits, Johannes Kepler University.

 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:

 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
*/
package at.jku.iic.pmd;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.util.Properties;

/**
 * @author bklepic
 *
 */
public class Parser {

	FieldConfig fieldCfg;
	
	public Parser(String file) {
		super();
		Properties global = new Properties();
		InputStream config = null;
		try {
			config = new FileInputStream(file);
			// load a properties file
			global.load(config);

			Properties settingsProp = new Properties();
			settingsProp.load(new StringReader(global.getProperty("settings").replace(';', '\n')));
			
			int flownum = Integer.parseInt(settingsProp.getProperty("flownum"));
			int srcnum = Integer.parseInt(settingsProp.getProperty("srcnum"));
			int boundx = Integer.parseInt(settingsProp.getProperty("boundx"));
			int boundy = Integer.parseInt(settingsProp.getProperty("boundy"));
			int boundz = Integer.parseInt(settingsProp.getProperty("boundz"));
			
			fieldCfg = new FieldConfig(flownum, srcnum, boundx, boundy, boundz);
			
			for (int i = 0; i < srcnum; i++) { 
				Properties srcProp = new Properties();
				srcProp.load(new StringReader(global.getProperty("src" + i).replace(';', '\n')));
				int srcx = Integer.parseInt(srcProp.getProperty("srcx"));
				int srcy = Integer.parseInt(srcProp.getProperty("srcy"));
				fieldCfg.setSource(i, srcx, srcy);
			}
			
			for (int i = 0; i < flownum; i++) { 
				Properties flowProp = new Properties();
				flowProp.load(new StringReader(global.getProperty("flow" + i).replace(';', '\n')));
				int length = Integer.parseInt(flowProp.getProperty("length"));
				int dirchnum = Integer.parseInt(flowProp.getProperty("dirchnum"));
				int sx = Integer.parseInt(flowProp.getProperty("sx"));
				int sy = Integer.parseInt(flowProp.getProperty("sy"));
				int ex = Integer.parseInt(flowProp.getProperty("ex"));
				int ey = Integer.parseInt(flowProp.getProperty("ey"));
				fieldCfg.setFlow(i, length, dirchnum, sx, sy, ex, ey);
			}
		} catch (IOException ex) {
			ex.printStackTrace();
		} finally {
			if (config != null) {
				try {
					config.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
   /* 
	public static void main(String[] args) {
		Parser p = new Parser("data.cfg");
		for (int i = 0; i < p.flowCfg.flownum; i++) {
			System.out.println("Flow" + i + 
					": length: " + p.flowCfg.flows[i].length + 
					", dirchnum: " + p.flowCfg.flows[i].dirchnum + 
					", srcx: " + p.flowCfg.flows[i].srcx + 
					", srcy: " + p.flowCfg.flows[i].srcy + 
					", sx: " + p.flowCfg.flows[i].sx + 
					", sy: " + p.flowCfg.flows[i].sy + 
					", ex: " + p.flowCfg.flows[i].ex + 
					", ey: " + p.flowCfg.flows[i].ey);
		}
    }
    */
}
