/*
 * Copyright (c) 2018 Institute for Integrated Circuits, Johannes Kepler University.

 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:

 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
*/
package at.jku.iic.pmd;
import com.microsoft.z3.Context;
import com.microsoft.z3.IntExpr;

/**
 * @author bklepic
 *
 */
public class Sample {
	
	Coords head, tail;
	Coords[] sample;
	Coords source;
	int id, timeStep, sampleLen;
	
	Coords[] dirChangers;
	int numDirChanges;

	public Sample(Context ctx, int id, int timeStep, int sampleLen, int numDirChanges) {
		super();
		this.id = id;
		this.timeStep = timeStep;
		this.sampleLen = sampleLen;
		this.head = new Coords(
			(IntExpr) ctx.mkConst(ctx.mkSymbol("S" + id + "t" + timeStep + "_head_x"), ctx.getIntSort()), 
			(IntExpr) ctx.mkConst(ctx.mkSymbol("S" + id + "t" + timeStep + "_head_y"), ctx.getIntSort())
		);
		this.tail = new Coords(
			(IntExpr) ctx.mkConst(ctx.mkSymbol("S" + id + "t" + timeStep + "_tail_x"), ctx.getIntSort()), 
			(IntExpr) ctx.mkConst(ctx.mkSymbol("S" + id + "t" + timeStep + "_tail_y"), ctx.getIntSort())
		);
		this.sample = new Coords[sampleLen];
		sample[0] = head;
		sample[sampleLen-1] = tail;
		for (int i = 1; i < sample.length - 1; i++) {
			sample[i] = new Coords(
				(IntExpr) ctx.mkConst(ctx.mkSymbol("S" + id + "t" + timeStep + "_" + i + "_x"), ctx.getIntSort()), 
				(IntExpr) ctx.mkConst(ctx.mkSymbol("S" + id + "t" + timeStep + "_" + i + "_y"), ctx.getIntSort())
			); 
		}
		this.numDirChanges = numDirChanges;
		dirChangers = new Coords[numDirChanges];
		for(int i = 0; i < this.dirChangers.length; i++){
			dirChangers[i] = new Coords(
				(IntExpr) ctx.mkConst(ctx.mkSymbol("S" + id + "t" + timeStep + "_dirChanger" + i + "_x"), ctx.getIntSort()), 
				(IntExpr) ctx.mkConst(ctx.mkSymbol("S" + id + "t" + timeStep + "_dirChanger" + i + "_y"), ctx.getIntSort())
			);
		}
		this.source = new Coords(
			(IntExpr) ctx.mkConst(ctx.mkSymbol("S" + id + "t" + timeStep + "_src_x"), ctx.getIntSort()), 
			(IntExpr) ctx.mkConst(ctx.mkSymbol("S" + id + "t" + timeStep + "_src_y"), ctx.getIntSort())
		);
	}
}
