/*
 * Copyright (c) 2018 Institute for Integrated Circuits, Johannes Kepler University.

 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:

 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
*/
package at.jku.iic.pmd.benchmark;

import java.awt.Point;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import at.jku.iic.pmd.TaskContainer;
import at.jku.iic.pmd.Grid;
import at.jku.iic.pmd.Sample;

/**
 * A hard coded IVD-benchmark. This benchmark was taken from the paper &quot;High-level synthesis of digital
 * microfluidic biochips&quot; and has been modified in order to be suitable for a PMD. For further details we
 * refer to our paper &quot;Sound Valve-Control for Programmable Microfluidic Devices&quot;.
 */
public class InVitro2 {

    public static List<TaskContainer> getBenchmark() {

        return Arrays.asList(getBio1(), getBio2(), getBio3(), getBio4(), getBio5(), getBio6(), getBio7(), getBio8(), getBio9(), getBio10(),
                getBio11(), getBio12(), getBio13(), getBio14(), getBio15());
    }

    public static TaskContainer getBio1() {

        List<Point> obstacles = Collections.emptyList();
        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(10, 1), new Point(11, 1)),
                Arrays.asList(new Point(5, 8), new Point(5, 7)));
        Sample s2 = Sample.createGridToGridSample(2, Arrays.asList(new Point(14, 5), new Point(14, 4)),
                Arrays.asList(new Point(5, 4), new Point(6, 4)));

        List<Sample> samples = Arrays.asList(s1, s2);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio2() {

        List<Point> obstacles = Grid.getPointsInRectangle(new Point(2, 7), new Point(4, 9));
        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(1, 5), new Point(1, 6)),
                Arrays.asList(new Point(3, 1), new Point(2, 1)));
        Sample s2 = Sample.createGridToGridSample(2, Arrays.asList(new Point(5, 4), new Point(5, 5)),
                Arrays.asList(new Point(4, 1), new Point(5, 1)));
        Sample s3 = Sample.createGridToGridSample(2, Arrays.asList(new Point(14, 10), new Point(14, 9)),
                Arrays.asList(new Point(5, 12), new Point(6, 12)));

        List<Sample> samples = Arrays.asList(s1, s2, s3);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio3() {

        List<Point> obstacles = Grid.getPointsInRectangle(new Point(2, 2), new Point(5, 5));
        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(5, 1), new Point(4, 1)),
                Arrays.asList(new Point(2, 8), new Point(2, 7)));
        Sample s2 = Sample.createGridToGridSample(2, Arrays.asList(new Point(1, 5), new Point(1, 4)),
                Arrays.asList(new Point(11, 3), new Point(12, 3)));
        Sample s3 = Sample.createGridToGridSample(2, Arrays.asList(new Point(5, 12), new Point(5, 13)),
                Arrays.asList(new Point(11, 4), new Point(12, 4)));
        Sample s4 = Sample.createGridToGridSample(2, Arrays.asList(new Point(10, 1), new Point(11, 1)),
                Arrays.asList(new Point(5, 8), new Point(5, 7)));
        Sample s5 = Sample.createGridToGridSample(2, Arrays.asList(new Point(1, 10), new Point(1, 11)),
                Arrays.asList(new Point(7, 9), new Point(6, 9)));

        List<Sample> samples = Arrays.asList(s1, s2, s3, s4, s5);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio4() {

        List<Point> obstacles = Grid.getPointsInRectangle(new Point(7, 2), new Point(10, 5));
        obstacles.addAll(Grid.getPointsInRectangle(new Point(2, 9), new Point(5, 13)));

        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(6, 4), new Point(6, 3)),
                Arrays.asList(new Point(12, 10), new Point(12, 9)));
        Sample s2 = Sample.createGridToGridSample(2, Arrays.asList(new Point(7, 9), new Point(7, 10)),
                Arrays.asList(new Point(8, 6), new Point(8, 7)));
        Sample s3 = Sample.createGridToGridSample(2, Arrays.asList(new Point(6, 13), new Point(6, 12)),
                Arrays.asList(new Point(8, 14), new Point(7, 14)));

        List<Sample> samples = Arrays.asList(s1, s2, s3);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio5() {

        List<Point> obstacles = Grid.getPointsInRectangle(new Point(7, 11), new Point(9, 13));
        obstacles.addAll(Grid.getPointsInRectangle(new Point(11, 11), new Point(13, 13)));

        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(9, 1), new Point(10, 1)),
                Arrays.asList(new Point(5, 4), new Point(6, 4)));
        Sample s2 = Sample.createGridToGridSample(2, Arrays.asList(new Point(2, 8), new Point(2, 7)),
                Arrays.asList(new Point(3, 10), new Point(3, 9)));
        Sample s3 = Sample.createGridToGridSample(2, Arrays.asList(new Point(8, 10), new Point(7, 10)),
                Arrays.asList(new Point(11, 5), new Point(11, 6)));
        Sample s4 = Sample.createGridToGridSample(2, Arrays.asList(new Point(14, 10), new Point(14, 11)),
                Arrays.asList(new Point(12, 5), new Point(12, 6)));
        Sample s5 = Sample.createGridToGridSample(2, Arrays.asList(new Point(1, 13), new Point(1, 12)),
                Arrays.asList(new Point(7, 14), new Point(6, 14)));

        List<Sample> samples = Arrays.asList(s1, s2, s3, s4, s5);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio6() {

        List<Point> obstacles = Grid.getPointsInRectangle(new Point(2, 3), new Point(4, 5));
        obstacles.addAll(Grid.getPointsInRectangle(new Point(7, 6), new Point(12, 9)));
        obstacles.addAll(Grid.getPointsInRectangle(new Point(7, 11), new Point(9, 13)));
        obstacles.addAll(Grid.getPointsInRectangle(new Point(11, 11), new Point(13, 13)));

        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(3, 14), new Point(4, 14)),
                Arrays.asList(new Point(2, 8), new Point(2, 9)));
        Sample s2 = Sample.createGridToGridSample(2, Arrays.asList(new Point(10, 1), new Point(11, 1)),
                Arrays.asList(new Point(5, 8), new Point(5, 7)));

        List<Sample> samples = Arrays.asList(s1, s2);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio7() {

        List<Point> obstacles = Grid.getPointsInRectangle(new Point(2, 3), new Point(4, 5));
        obstacles.addAll(Grid.getPointsInRectangle(new Point(2, 9), new Point(5, 13)));
        obstacles.addAll(Grid.getPointsInRectangle(new Point(7, 11), new Point(9, 13)));
        obstacles.addAll(Grid.getPointsInRectangle(new Point(11, 11), new Point(13, 13)));

        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(13, 7), new Point(14, 7)),
                Arrays.asList(new Point(10, 8), new Point(11, 8)));

        List<Sample> samples = Arrays.asList(s1);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio8() {

        List<Point> obstacles = Collections.emptyList();

        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(2, 8), new Point(2, 9)),
                Arrays.asList(new Point(3, 6), new Point(3, 7)));
        Sample s2 = Sample.createGridToGridSample(2, Arrays.asList(new Point(6, 8), new Point(7, 8)),
                Arrays.asList(new Point(5, 4), new Point(5, 5)));
        Sample s3 = Sample.createGridToGridSample(2, Arrays.asList(new Point(6, 10), new Point(6, 9)),
                Arrays.asList(new Point(5, 12), new Point(5, 11)));
        Sample s4 = Sample.createGridToGridSample(2, Arrays.asList(new Point(1, 4), new Point(1, 3)),
                Arrays.asList(new Point(7, 14), new Point(6, 14)));
        Sample s5 = Sample.createGridToGridSample(2, Arrays.asList(new Point(12, 14), new Point(13, 14)),
                Arrays.asList(new Point(8, 14), new Point(9, 14)));
        Sample s6 = Sample.createGridToGridSample(2, Arrays.asList(new Point(14, 5), new Point(14, 4)),
                Arrays.asList(new Point(9, 6), new Point(10, 6)));
        Sample s7 = Sample.createGridToGridSample(2, Arrays.asList(new Point(10, 12), new Point(10, 13)),
                Arrays.asList(new Point(9, 7), new Point(10, 7)));

        List<Sample> samples = Arrays.asList(s1, s2, s3, s4, s5, s6, s7);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio9() {

        List<Point> obstacles = Grid.getPointsInRectangle(new Point(2, 3), new Point(4, 5));
        obstacles.addAll(Grid.getPointsInRectangle(new Point(8, 8), new Point(11, 13)));

        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(3, 6), new Point(2, 6)),
                Arrays.asList(new Point(7, 1), new Point(7, 2)));
        Sample s2 = Sample.createGridToGridSample(2, Arrays.asList(new Point(14, 5), new Point(14, 6)),
                Arrays.asList(new Point(8, 1), new Point(8, 2)));
        Sample s3 = Sample.createGridToGridSample(2, Arrays.asList(new Point(5, 12), new Point(5, 13)),
                Arrays.asList(new Point(3, 10), new Point(4, 10)));
        Sample s4 = Sample.createGridToGridSample(2, Arrays.asList(new Point(14, 10), new Point(14, 11)),
                Arrays.asList(new Point(3, 9), new Point(4, 9)));

        List<Sample> samples = Arrays.asList(s1, s2, s3, s4);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio10() {

        List<Point> obstacles = Grid.getPointsInRectangle(new Point(2, 3), new Point(4, 5));
        obstacles.addAll(Grid.getPointsInRectangle(new Point(6, 2), new Point(9, 6)));
        obstacles.addAll(Grid.getPointsInRectangle(new Point(2, 10), new Point(6, 13)));

        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(12, 10), new Point(12, 9)),
                Arrays.asList(new Point(12, 14), new Point(12, 13)));

        List<Sample> samples = Arrays.asList(s1);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio11() {

        List<Point> obstacles = Grid.getPointsInRectangle(new Point(2, 10), new Point(6, 13));
        obstacles.addAll(Grid.getPointsInRectangle(new Point(6, 2), new Point(9, 6)));
        obstacles.addAll(Grid.getPointsInRectangle(new Point(11, 11), new Point(13, 13)));

        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(3, 6), new Point(3, 5)),
                Arrays.asList(new Point(7, 14), new Point(7, 13)));

        List<Sample> samples = Arrays.asList(s1);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio12() {

        List<Point> obstacles = Grid.getPointsInRectangle(new Point(11, 11), new Point(13, 13));

        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(1, 11), new Point(1, 12)),
                Arrays.asList(new Point(1, 8), new Point(1, 9)));
        Sample s2 = Sample.createGridToGridSample(2, Arrays.asList(new Point(5, 4), new Point(5, 5)),
                Arrays.asList(new Point(5, 3), new Point(6, 3)));

        List<Sample> samples = Arrays.asList(s1, s2);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio13() {

        List<Point> obstacles = Grid.getPointsInRectangle(new Point(2, 3), new Point(4, 5));

        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(5, 8), new Point(4, 8)),
                Arrays.asList(new Point(12, 10), new Point(11, 10)));
        Sample s2 = Sample.createGridToGridSample(2, Arrays.asList(new Point(12, 14), new Point(13, 14)),
                Arrays.asList(new Point(7, 14), new Point(8, 14)));

        List<Sample> samples = Arrays.asList(s1, s2);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio14() {

        List<Point> obstacles = Grid.getPointsInRectangle(new Point(11, 11), new Point(13, 13));

        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(3, 6), new Point(3, 5)),
                Arrays.asList(new Point(7, 14), new Point(7, 13)));

        List<Sample> samples = Arrays.asList(s1);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    public static TaskContainer getBio15() {

        List<Point> obstacles = Collections.emptyList();
        List<Point> inputs = Arrays.asList(new Point(1, 0), new Point(14, 15));
        List<Point> exits = Arrays.asList(new Point(14, 0), new Point(1, 15));

        Sample s1 = Sample.createGridToGridSample(2, Arrays.asList(new Point(12, 14), new Point(13, 14)),
                Arrays.asList(new Point(7, 14), new Point(8, 14)));

        List<Sample> samples = Arrays.asList(s1);
        Grid grid = new Grid(14, obstacles, inputs, exits, samples);

        return new TaskContainer(grid, samples);
    }

    private InVitro2() {

    }
}
