/*
 * Copyright (c) 2018 Institute for Integrated Circuits, Johannes Kepler University.

 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:

 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
*/
package at.jku.iic.pmd.main;

import java.io.IOException;
import java.nio.file.Paths;

import at.jku.iic.pmd.TaskContainer;
import at.jku.iic.pmd.mazerouter.MazeRouterCaller;
import at.jku.iic.pmd.parser.RoutabilityDrivenBenchmarkParser;

/**
 * Executes the benchmark file located at the path provided as command line argument.
 */
public class BenchmarkExecutor {

    public static void main(String[] args) throws IOException {

        if (args.length != 1) {
            throw new IllegalArgumentException("Please provide the path to the benchmark file.");
        }

        RoutabilityDrivenBenchmarkParser parser = new RoutabilityDrivenBenchmarkParser();
        TaskContainer benchmark = parser.readBenchmark(Paths.get(args[0]));

        if (new MazeRouterCaller().executeRouting(benchmark.getGrid(), benchmark.getSamples())) {
            System.out.println(benchmark.getGrid().getLatestTimeOfGridChange());
        } else {
            System.out.println("No valid routing found!");
        }
    }
}
