/*
 * Copyright (c) 2018 Institute for Integrated Circuits, Johannes Kepler University.

 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:

 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
*/
package at.jku.iic.pmd.main;

import java.io.IOException;
import java.util.List;

import at.jku.iic.pmd.TaskContainer;
import at.jku.iic.pmd.benchmark.InVitro1;
import at.jku.iic.pmd.benchmark.InVitro2;
import at.jku.iic.pmd.mazerouter.MazeRouterCaller;

/**
 * Executes an InVitro benchmark. The command line argument
 * <li>&quot;1&quot; is used to select InVitro1 or
 * <li>&quot;2&quot; is used to select InVitro2.
 *
 */
public class InVitroExecutor {

    public static void main(String[] args) throws IOException {

        if (args.length != 1) {
            throw new IllegalArgumentException("Please provide the benchmark ID. Supported IDs are \"1\" and \"2\".");
        }

        MazeRouterCaller router = new MazeRouterCaller();

        int id = Integer.parseInt(args[0]);
        List<TaskContainer> benchmark;
        if (id == 1) benchmark = InVitro1.getBenchmark();
        else if (id == 2) benchmark = InVitro2.getBenchmark();
        else throw new UnsupportedOperationException("Unknown id " + args[0]);

        int totalTime = 0;
        for (TaskContainer subPart : benchmark) {

            if (!router.executeRouting(subPart.getGrid(), subPart.getSamples())) {
                System.out.println("No valid routing found!");
                return;
            }

            totalTime += subPart.getGrid().getLatestTimeOfGridChange();
        }
        System.out.println(totalTime);
    }
}
