/*
 * Copyright (c) 2018 Institute for Integrated Circuits, Johannes Kepler University.

 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:

 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
*/
package at.jku.iic.pmd.parser;

import java.awt.Point;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import at.jku.iic.pmd.TaskContainer;
import at.jku.iic.pmd.Grid;
import at.jku.iic.pmd.Sample;

/**
 * A simple parser for the benchmarks provided in &quot;A routability-driven flow routing algorithm for
 * programmable microfluidic devices&quot;.
 *
 */
public class RoutabilityDrivenBenchmarkParser {

    public TaskContainer readBenchmark(Path file) throws IOException {

        Iterator<String> lineIt = Files.readAllLines(file).iterator();
        String[] tokens = lineIt.next().trim().replaceAll("  ", " ").split(" ");

        int mapDimension = Integer.parseInt(tokens[3]) + 1;

        List<Point> inputs = new LinkedList<>();
        List<Point> exits = new LinkedList<>();

        List<Sample> samples = new LinkedList<>();

        while (lineIt.hasNext()) {
            tokens = lineIt.next().trim().replaceAll("  ", " ").split(" ");

            int length = Integer.parseInt(tokens[2]);
            int sx = Integer.parseInt(tokens[4]);
            int sy = Integer.parseInt(tokens[6]);
            int ex = Integer.parseInt(tokens[8]);
            int ey = Integer.parseInt(tokens[10]);

            Point input = new Point(mapCoordinateToExternalConnector(sx, sy, mapDimension));
            Point exit = new Point(mapCoordinateToExternalConnector(ex, ey, mapDimension));

            inputs.add(input);
            exits.add(exit);

            samples.add(Sample.createInputToExitSample(length, input, exit));
        }

        Grid grid = new Grid(mapDimension, Collections.emptyList(), inputs, exits, samples);
        return new TaskContainer(grid, samples);
    }

    private Point mapCoordinateToExternalConnector(int x, int y, int mapDimension) {

        if (y == 0 && x < mapDimension - 1) {
            // North
            return new Point(x + 1, y);
        } else if (x == mapDimension - 1 && y < mapDimension - 1) {
            // East
            return new Point(x + 2, y + 1);
        } else if (y == mapDimension - 1 && x > 0) {
            // South
            return new Point(x + 1, y + 1);
        }
        // West
        assert x == 0 && y > 0;
        return new Point(x, y + 1);
    }
}
